@@ -1,18 +1,13 @@
-//@line 40 "/builds/tinderbox/Tb-Mozilla1.8.0-Release/Linux_2.4.18-14_Depend/mozilla/mail/base/content/mailWindow.js"
+//@line 40 "/builds/tinderbox/Tb-Mozilla1.8-Release/Linux_2.4.18-14_Depend/mozilla/mail/base/content/mailWindow.js"
 
 //This file stores variables common to mail windows
 var messengerContractID        = "@mozilla.org/messenger;1";
 var statusFeedbackContractID   = "@mozilla.org/messenger/statusfeedback;1";
 var mailSessionContractID      = "@mozilla.org/messenger/services/session;1";
-var secureUIContractID         = "@mozilla.org/secure_browser_ui;1";
-
-
-var prefContractID             = "@mozilla.org/preferences-service;1";
 var msgWindowContractID      = "@mozilla.org/messenger/msgwindow;1";
 
 var messenger;
 var pref;
-var prefServices;
 var statusFeedback;
 var messagePaneController;
 var msgWindow;
@@ -31,26 +26,23 @@ var gBrandBundle;
 var datasourceContractIDPrefix = "@mozilla.org/rdf/datasource;1?name=";
 var accountManagerDSContractID = datasourceContractIDPrefix + "msgaccountmanager";
 var folderDSContractID         = datasourceContractIDPrefix + "mailnewsfolders";
-
+var unreadFoldersDSContractID = datasourceContractIDPrefix + "mailnewsunreadfolders";
+var favoriteFoldersDSContractID = datasourceContractIDPrefix + "mailnewsfavefolders";
+var recentFoldersDSContractID = datasourceContractIDPrefix + "mailnewsrecentfolders";
 var accountManagerDataSource;
 var folderDataSource;
+var unreadFolderDataSource;
+var favoriteFoldersDataSource;
+var recentFoldersDataSource;
 
 var accountCentralBox = null;
-var gSearchBox = null;
-var gAccountCentralLoaded = false;
+var gAccountCentralLoaded = true;
 var gFakeAccountPageLoaded = false;
 //End progress and Status variables
 
-// for checking if the folder loaded is Draft or Unsent which msg is editable
-var gIsEditableMsgFolder = false;
-var gOfflineManager;
-
-// cache the last keywords
-var gLastKeywords = "";
-
 function OnMailWindowUnload()
 {
-  RemoveMailOfflineObserver();
+  MailOfflineMgr.uninit();
   ClearPendingReadTimer();
 
   var searchSession = GetSearchSession();
@@ -79,12 +71,16 @@ function OnMailWindowUnload()
   mailSession.RemoveMsgWindow(msgWindow);
   messenger.SetWindow(null, null);
 
-  var msgDS = folderDataSource.QueryInterface(Components.interfaces.nsIMsgRDFDataSource);
-  msgDS.window = null;
-
-  msgDS = accountManagerDataSource.QueryInterface(Components.interfaces.nsIMsgRDFDataSource);
-  msgDS.window = null;
-
+  var msgDS;
+  var viewDataSources = [accountManagerDataSource, folderDataSource, 
+                        unreadFolderDataSource, favoriteFoldersDataSource,
+                        recentFoldersDataSource];
+                        
+  for (index in viewDataSources)
+  {
+    msgDS = viewDataSources[index].QueryInterface(Components.interfaces.nsIMsgRDFDataSource);
+    msgDS.window = null;
+  }
 
   msgWindow.closeWindow();
 }
@@ -100,8 +96,8 @@ function CreateMailWindowGlobals()
   // get the messenger instance
   CreateMessenger();
 
-  prefServices = Components.classes[prefContractID].getService(Components.interfaces.nsIPrefService);
-  pref = prefServices.getBranch(null);
+  pref = Components.classes["@mozilla.org/preferences-service;1"]
+          .getService(Components.interfaces.nsIPrefBranch2);
 
   //Create windows status feedback
   // set the JS implementation of status feedback before creating the c++ one..
@@ -165,9 +161,9 @@ function CreateMailWindowGlobals()
   //Create datasources
   accountManagerDataSource = Components.classes[accountManagerDSContractID].createInstance();
   folderDataSource         = Components.classes[folderDSContractID].getService();
-
-  accountCentralBox = document.getElementById("accountCentralBox");
-  gSearchBox = document.getElementById("searchBox");
+  unreadFolderDataSource = Components.classes[unreadFoldersDSContractID].getService();
+  favoriteFoldersDataSource = Components.classes[favoriteFoldersDSContractID].getService();
+  recentFoldersDataSource = Components.classes[recentFoldersDSContractID].getService();
 }
 
 function InitMsgWindow()
@@ -177,8 +173,9 @@ function InitMsgWindow()
   msgWindow.msgHeaderSink = messageHeaderSink;
   msgWindow.SetDOMWindow(window);
   mailSession.AddMsgWindow(msgWindow);
-  document.getElementById("messagepane").docShell.allowAuth = false;
+  getBrowser().docShell.allowAuth = false;
   msgWindow.rootDocShell.allowAuth = true; 
+  msgWindow.rootDocShell.appType = Components.interfaces.nsIDocShell.APP_TYPE_MAIL;
 }
 
 function AddDataSources()
@@ -202,12 +199,15 @@ function AddDataSources()
 
   //Add statusFeedback
 
-  var msgDS = folderDataSource.QueryInterface(Components.interfaces.nsIMsgRDFDataSource);
-  msgDS.window = msgWindow;
-
-  msgDS = accountManagerDataSource.QueryInterface(Components.interfaces.nsIMsgRDFDataSource);
-  msgDS.window = msgWindow;
-
+  var msgDS;
+  var viewDataSources = [accountManagerDataSource, folderDataSource, 
+                        unreadFolderDataSource, favoriteFoldersDataSource,
+                        recentFoldersDataSource];
+  for (index in viewDataSources)
+  {
+    msgDS = viewDataSources[index].QueryInterface(Components.interfaces.nsIMsgRDFDataSource);
+    msgDS.window = msgWindow;
+  }
 }
 
 function SetupMoveCopyMenus(menuid, accountManagerDataSource, folderDataSource)
@@ -337,6 +337,7 @@ nsMsgStatusFeedback.prototype =
         gTimelineService.markTimer("FolderLoading");
         gTimelineService.resetTimer("FolderLoading");
       }
+
       this.ensureStatusFields();
       this.showStatusString(defaultStatus);
       
@@ -448,125 +449,139 @@ function StopUrls()
   msgWindow.StopUrls();
 }
 
+/** 
+ * @returns the pref name to use for fetching the start page url. Every time the application version changes, 
+ * return "mailnews.start_page.override_url". If this is the first time the application has been 
+ * launched, return "mailnews.start_page.welcome_url". Otherwise return "mailnews.start_page.url".
+ */
+function startPageUrlPref()
+{
+  var prefForStartPageUrl = "mailnews.start_page.url";
+  var savedVersion = null;
+  try {
+    savedVersion = pref.getCharPref("mailnews.start_page_override.mstone");
+  } catch (ex) {}
+  
+  if (savedVersion != "ignore")
+  {
+    var currentPlatformVersion = Components.classes["@mozilla.org/xre/app-info;1"].
+                                            getService(Components.interfaces.nsIXULAppInfo).platformVersion;
+    pref.setCharPref("mailnews.start_page_override.mstone", currentPlatformVersion);
+    // Use the welcome URL the first time we run
+    if (!savedVersion)
+      prefForStartPageUrl = "mailnews.start_page.welcome_url";
+    else if (currentPlatformVersion != savedVersion)
+      prefForStartPageUrl = "mailnews.start_page.override_url";
+  }
+  
+  return prefForStartPageUrl;
+}
+
 function loadStartPage() 
 {
-  try 
+  try
   {
     gMessageNotificationBar.clearMsgNotifications();
     
     var startpageenabled = pref.getBoolPref("mailnews.start_page.enabled");
-    if (startpageenabled) 
+    // only load the start page if we are online
+    if (startpageenabled && MailOfflineMgr.isOnline())
     {
-      
-      var startpage = pref.getComplexValue("mailnews.start_page.url", Components.interfaces.nsIPrefLocalizedString).data;
-
-      // Some users have our old default start page
-      // showing up as a user pref instead of a default pref. If this is the case, clear the user pref by hand 
-      // and re-read it again so we get the correct default start page.
-      if (startpage == "chrome://messenger/locale/start.html")
-      {
-        pref.clearUserPref("mailnews.start_page.url");
-        startpage = pref.getComplexValue("mailnews.start_page.url", Components.interfaces.nsIPrefLocalizedString).data;
-      }
-
-      if (startpage != "") 
+      var startpage = getFormattedRegionURL(startPageUrlPref());
+      if (startpage)
       {
         GetMessagePaneFrame().location.href = startpage;
-        //dump("start message pane with: " + startpage + "\n");
         ClearMessageSelection();
       }
     }
   }
-  catch (ex) 
+  catch (ex)
   {
     dump("Error loading start page.\n");
     return;
   }
 }
 
+// When the ThreadPane is hidden via the displayDeck, we should collapse the
+// elements that are only meaningful to the thread pane. When AccountCentral is
+// shown via the displayDeck, we need to switch the displayDeck to show the
+// accountCentralBox, and load the iframe in the AccountCentral box with
+// corresponding page.
 function ShowAccountCentral()
 {
-  if (document.getElementById("displayDeck").selectedPanel == accountCentralBox)
-    ShowingAccountCentral(); // force us to reload account central because the user has switched accounts
-  else
+  try
+  {
     document.getElementById("displayDeck").selectedPanel = accountCentralBox;
+    var acctCentralPage = pref.getComplexValue("mailnews.account_central_page.url",
+                                               Components.interfaces.nsIPrefLocalizedString).data;
+    window.frames["accountCentralPane"].location.href = acctCentralPage;
+  }
+  catch (ex)
+  {
+    dump("Error loading AccountCentral page -> " + ex + "\n");
+  }
 }
 
-// When AccountCentral is shown via the displayDeck, we need to switch the
-// displayDeck to show the accountCentralBox, collapse all the other
-// UI elements that aren't meaningful for AccountCentral, and finally
-// load the iframe in the AccountCentral box with corresponding page.
 function ShowingAccountCentral()
 {
-    try
-    {
-        var acctCentralPage = pref.getComplexValue("mailnews.account_central_page.url",
-                                               Components.interfaces.nsIPrefLocalizedString).data;
-        GetMessagePane().collapsed = true;
-        document.getElementById("threadpane-splitter").collapsed = true;
-        gSearchBox.collapsed = true;
+  if (!IsFolderPaneCollapsed())
+    GetFolderTree().focus();        
 
-        window.frames["accountCentralPane"].location.href = acctCentralPage;
-        
-        if (!IsFolderPaneCollapsed())
-            GetFolderTree().focus();        
-
-        gAccountCentralLoaded = true;
-    }
-    catch (ex)
-    {
-        dump("Error loading AccountCentral page -> " + ex + "\n");
-    }
+  gAccountCentralLoaded = true;
 }
 
 function HidingAccountCentral()
 {
-    try
-    {
-        window.frames["accountCentralPane"].location.href = "about:blank";
-    }
-    catch (ex)
-    {
-        dump("Error hiding AccountCentral page -> " + ex + "\n");
-    }
-    gAccountCentralLoaded = false;
+  gAccountCentralLoaded = false;
 }
 
 function ShowThreadPane()
 {
-    document.getElementById("displayDeck").selectedPanel = 
-        document.getElementById("threadPaneBox");
+  document.getElementById("displayDeck").selectedPanel = 
+    document.getElementById("threadPaneBox");
 }
 
 function ShowingThreadPane()
 {
-    gSearchBox.collapsed = false;
-    var threadPaneSplitter = document.getElementById("threadpane-splitter");
-    threadPaneSplitter.collapsed = false;
-    GetMessagePane().collapsed =
-        (threadPaneSplitter.getAttribute("state") == "collapsed");
+  var threadPaneSplitter = document.getElementById("threadpane-splitter");
+  threadPaneSplitter.collapsed = false;
+  GetMessagePane().collapsed = (threadPaneSplitter.getAttribute("state") == "collapsed");
+  // XXX We need to force the tree to refresh its new height
+  // so that it will correctly scroll to the newest message
+  GetThreadTree().boxObject.height;  
+  document.getElementById("key_toggleMessagePane").removeAttribute("disabled");
 }
 
 function HidingThreadPane()
 {
-    ClearThreadPane();
+  ClearThreadPane();
+  GetUnreadCountElement().hidden = true;
+  GetTotalCountElement().hidden = true;
+  GetMessagePane().collapsed = true;
+  document.getElementById("threadpane-splitter").collapsed = true;
+  document.getElementById("key_toggleMessagePane").setAttribute("disabled", "true");
+}
+
+// the find toolbar needs a method called getBrowser
+function getBrowser() 
+{
+  return document.getElementById("messagepane");
 }
 
 function ObserveDisplayDeckChange(event)
 {
-    var deck = document.getElementById("displayDeck");
-    var nowSelected = null;
-    try { nowSelected = deck.selectedPanel.id; } catch (ex) { }
+  var selectedPanel = document.getElementById("displayDeck").selectedPanel;
+  var nowSelected = selectedPanel ? selectedPanel.id : "";
 
-    if (nowSelected == "threadPaneBox")
-        ShowingThreadPane();
-    else
-        HidingThreadPane();
-    
-    if (nowSelected == "accountCentralBox")
-        ShowingAccountCentral();
-    else
-        HidingAccountCentral();
+  if (nowSelected == "threadPaneBox")
+    ShowingThreadPane();
+  else
+    HidingThreadPane();
+  
+  if (nowSelected == "accountCentralBox")
+    ShowingAccountCentral();
+  else
+    HidingAccountCentral();
 }
 
 // Given the server, open the twisty and the set the selection
@@ -574,23 +589,20 @@ function ObserveDisplayDeckChange(event)
 // prompt if offline.
 function OpenInboxForServer(server)
 {
-    try {
-        ShowThreadPane();
-        var inboxFolder = GetInboxFolder(server);
-        SelectFolder(inboxFolder.URI);
+  try {
+    ShowThreadPane();
+    var inboxFolder = GetInboxFolder(server);
+    SelectFolder(inboxFolder.URI);
 
-        if(CheckOnline())	{
-            if (server.type != "imap")
-                GetMessagesForInboxOnServer(server);
-        }
-        else if (DoGetNewMailWhenOffline()) {
-                GetMessagesForInboxOnServer(server);
-        }
-    }
-    catch (ex) {
-        dump("Error opening inbox for server -> " + ex + "\n");
-        return;
+    if (MailOfflineMgr.isOnline() || MailOfflineMgr.getNewMail())	{
+      if (server.type != "imap")
+        GetMessagesForInboxOnServer(server);
     }
+  }
+  catch (ex) {
+      dump("Error opening inbox for server -> " + ex + "\n");
+      return;
+  }
 }
 
 function GetSearchSession()
@@ -600,43 +612,3 @@ function GetSearchSession()
   else
     return null;
 }
-
-function SetKeywords(aKeywords) 
-{ 
-  // we cache the last keywords.  
-  // if there is no chagne, we do nothing.
-  // most of the time, this will be the case.
-  if (aKeywords == gLastKeywords)
-    return;
-
-  // these are the UI elements who care about keywords
-  var elements = document.getElementsByAttribute("keywordrelated","true");
-  var len = elements.length;
-  for (var i=0; i<len; i++) {
-    var element = elements[i];
-    var originalclass = element.getAttribute("originalclass");
-
-    // we use XBL for certain headers.
-    // if the element has keywordrelated="true"
-    // but no original class, it's an XBL widget
-    // so to get the real element, use getAnonymousElementByAttribute()
-    if (!originalclass) {
-      element = document.getAnonymousElementByAttribute(element, "keywordrelated", "true");
-      originalclass = element.getAttribute("originalclass");
-    }
-
-    if (aKeywords) {
-      if (element.getAttribute("appendoriginalclass") == "true") {
-        aKeywords += " " + originalclass;
-      }
-      element.setAttribute("class", aKeywords);
-    }
-    else {
-      // if no keywords, reset class to the original class
-      element.setAttribute("class", originalclass);
-    }
-  }
-
-  // cache the keywords 
-  gLastKeywords = aKeywords;
-}
